/* (C) 2022 YiRing, Inc. */
package com.yiring.common.util;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.file.FileReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.attribute.BasicFileAttributes;
import javax.servlet.http.HttpServletResponse;
import lombok.experimental.UtilityClass;
import org.apache.tomcat.util.http.fileupload.IOUtils;
import org.springframework.http.HttpHeaders;

/**
 * 文件工具类
 *
 * @author Jim
 * @version 0.1
 * 2022/4/22 13:54
 */

@UtilityClass
public class FileUtils {

    /**
     * 文件下载
     * @param response HttpServletResponse
     * @param file File
     * @throws IOException IOException
     */
    public void download(HttpServletResponse response, File file) throws IOException {
        String filename = URLEncoder.encode(file.getName(), StandardCharsets.UTF_8);
        BasicFileAttributes basicFileAttributes = Files.readAttributes(file.toPath(), BasicFileAttributes.class);
        long lastModified = basicFileAttributes.lastModifiedTime().toMillis();
        response.setHeader(HttpHeaders.LAST_MODIFIED, String.valueOf(lastModified));
        response.setHeader(HttpHeaders.CONTENT_LENGTH, String.valueOf(basicFileAttributes.size()));
        response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename=" + filename);
        response.setContentType(FileUtil.getMimeType(file.toPath()));
        FileReader.create(file).writeToStream(response.getOutputStream(), true);
    }

    /**
     * 文件下载
     * @param response HttpServletResponse
     * @param object 文件流
     * @param length 文件大小
     * @param filename 文件名称（带后缀）
     * @param contentType 文档类型
     * @param lastModified 最后修改时间
     * @throws IOException IOException
     */
    public void download(
        HttpServletResponse response,
        InputStream object,
        long length,
        String filename,
        String contentType,
        long lastModified
    ) throws IOException {
        filename = URLEncoder.encode(filename, StandardCharsets.UTF_8);
        response.setHeader(HttpHeaders.LAST_MODIFIED, String.valueOf(lastModified));
        response.setHeader(HttpHeaders.CONTENT_LENGTH, String.valueOf(length));
        response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename=" + filename);
        response.setContentType(contentType);
        IOUtils.copy(object, response.getOutputStream());
        object.close();
    }
}
